@@ -5,7 +5,7 @@
 
 
 setClass("GPos",
-    contains="GRanges",
+    contains=c("GenomicPos", "GRanges"),
     representation(
         ranges="IPos"
     )
@@ -16,17 +16,6 @@ setClass("GPos",
 ### Getters
 ###
 
-setMethod("names", "GPos", function(x) NULL)
-
-setReplaceMethod("names", "GPos",
-    function(x, value)
-    {
-        if (!is.null(value))
-            stop(class(x), " objects don't accept names")
-        x
-    }
-)
-
 setMethod("pos", "GPos", function(x) pos(ranges(x)))
 
 
@@ -34,7 +23,7 @@ setMethod("pos", "GPos", function(x) pos(ranges(x)))
 ### Collapse runs of "stitchable genomic ranges"
 ###
 ### 2 genomic ranges are "stitchable" if, in addition to be stitchable from
-### an integer ranges point-of-view (see stitch_Ranges() in
+### an integer ranges point-of-view (see stitch_IntegerRanges() in
 ### IRanges/R/IPos-class.R for what that means), they are also on the same
 ### chromosome and strand.
 
@@ -46,8 +35,8 @@ setMethod("pos", "GPos", function(x) pos(ranges(x)))
 ### of the elements in the input vector. It's also idempotent like range(),
 ### reduce(), and disjoin() (gaps() is not).
 
-### TODO: Define and export stitch() generic and method for Ranges objects
-### in the IRanges package (in inter-range-methods.R). Then make
+### TODO: Define and export stitch() generic and method for IntegerRanges
+### objects in the IRanges package (in inter-range-methods.R). Then make
 ### stitch_GenomicRanges() and stitch_GPos() the "stitch" methods for
 ### GenomicRanges and GPos objects, respectively, and support the
 ### 'ignore.strand' argument.
@@ -221,24 +210,24 @@ setMethod("as.data.frame", "GPos",
 }
 
 setMethod("updateObject", "GPos",
-   function(object, ..., verbose=FALSE)
-   {
-       version <- .get_GPos_version(object)
-       if (version == "current") {
-           if (verbose)
-               message("[updateObject] Internal representation of ",
-                       class(object), " object is current.\n",
-                       "[updateObject] Nothing to update.")
-           return(object)
-       }
-       if (verbose)
-           message("[updateObject] ", class(object), " object uses ",
-                   "internal representation from GenomicRanges\n",
-                   "[updateObject] ", version, ". Updating it ...")
-       object <- GPos(object@pos_runs)
-       metadata(object) <- metadata(object)
-       object
-   }
+    function(object, ..., verbose=FALSE)
+    {
+        version <- .get_GPos_version(object)
+        if (version == "current") {
+            if (verbose)
+                message("[updateObject] Internal representation of ",
+                        class(object), " object is current.\n",
+                        "[updateObject] Nothing to update.")
+            return(object)
+        }
+        if (verbose)
+            message("[updateObject] ", class(object), " object uses ",
+                    "internal representation from GenomicRanges\n",
+                    "[updateObject] ", version, ". Updating it ...")
+        object <- GPos(object@pos_runs)
+        metadata(object) <- metadata(object)
+        object
+    }
 )
 
 
@@ -270,7 +259,7 @@ show_GPos <- function(x, margin="",
         stop(class(x), " object uses internal representation from ",
              "GenomicRanges ", version, "\n  and cannot be displayed or ",
              "used. Please update it with:\n",
-             "    x <- updateObject(x, verbose=TRUE)")
+             "    object <- updateObject(object, verbose=TRUE)")
     x_class <- class(x)
     x_len <- length(x)
     x_mcols <- mcols(x)
