@@ -12,7 +12,6 @@ setClass("GRanges",
         seqnames="Rle",
         ranges="IRanges_OR_IPos",  # an IPos only for GPos
         strand="Rle",
-        elementMetadata="DataFrame",
         seqinfo="Seqinfo"
     ),
     prototype(
@@ -23,14 +22,20 @@ setClass("GRanges",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### "update" method
+### parallelSlotNames()
 ###
 
-### Having update() redirect to BiocGenerics:::replaceSlots() on GRanges
-### objects makes all the methods for GenomicRanges objects defined in
-### R/GenomicRanges-class.R work on GRanges objects.
-setMethod("update", "GRanges",
-    function(object, ...) BiocGenerics:::replaceSlots(object, ...)
+### Combine the new parallel slots with those of the parent class. Make sure
+### to put the new parallel slots *first*.
+setMethod("parallelSlotNames", "GRanges",
+    #function(x) c("seqnames", "ranges", "strand", callNextMethod())
+
+    ## TEMPORARY DEFINITION.
+    ## TODO: Remove this temporary definition and add "parallelSlotNames"
+    ## methods to all packages that define "extraColumnSlotNames" methods
+    ## (e.g. VariantAnnotation, GenomicTuples, InteractionSet, SGSeq).
+    function(x) c(extraColumnSlotNames(x), "seqnames", "ranges", "strand",
+                  callNextMethod())
 )
 
 
@@ -61,6 +66,60 @@ setMethod("update", "GRanges",
 setValidity2("GRanges", .valid.GRanges)
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### updateObject()
+###
+### Internal representation of GRanges objects has changed in GenomicRanges
+### 1.31.16 (Bioc 3.7).
+###
+
+.get_GRanges_version <- function(object)
+{
+    if (.hasSlot(object, "elementType")) "current" else "< 1.31.16"
+}
+
+setMethod("updateObject", "GRanges",
+    function(object, ..., verbose=FALSE)
+    {
+        ## elementType slot.
+        version <- .get_GRanges_version(object)
+        if (version == "current") {
+            if (verbose)
+                message("[updateObject] Internal representation of ",
+                        class(object), " object is current.\n",
+                        "[updateObject] Nothing to update.")
+        } else {
+            if (verbose)
+                message("[updateObject] ", class(object), " object uses ",
+                        "internal representation from\n",
+                        "[updateObject] GenomicRanges ", version, ". ",
+                        "Updating it ...")
+            object@elementType <- new(class(object))@elementType
+        }
+        ## ranges slot.
+        object@ranges <- updateObject(object@ranges, ..., verbose=verbose)
+        object
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### "update" method
+###
+
+### Having update() redirect to BiocGenerics:::replaceSlots() on GRanges
+### objects makes all the methods for GenomicRanges objects defined in
+### R/GenomicRanges-class.R work on GRanges objects.
+setMethod("update", "GRanges",
+    function(object, ...)
+    {
+        ## Fix old GRanges instances on-the-fly.
+        object <- updateObject(object, check=FALSE)
+        BiocGenerics:::replaceSlots(object, ...)
+    }
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructor
 ###
@@ -183,31 +242,6 @@ GRanges <- function(seqnames=NULL, ranges=NULL, strand=NULL,
                            mcols=mcols, seqlengths=seqlengths, seqinfo=seqinfo)
 }
 
-setMethod("updateObject", "GRanges",
-    function(object, ..., verbose=FALSE)
-    {
-        if (verbose)
-            message("updateObject(object = 'GRanges')")
-        if (is(try(object@seqinfo, silent=TRUE), "try-error")) {
-            object <- new(class(object),
-                          seqnames = object@seqnames,
-                          ranges = object@ranges,
-                          strand = object@strand,
-                          elementMetadata = object@elementMetadata,
-                          metadata = object@metadata,
-                          seqinfo = Seqinfo(seqnames = names(object@seqlengths),
-                                            seqlengths = object@seqlengths))
-            return(object)
-        }
-        if (is(try(validObject(object@seqinfo, complete=TRUE), silent=TRUE),
-               "try-error")) {
-            object@seqinfo <- updateObject(object@seqinfo)
-            return(object)
-        }
-        object
-    }
-)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Accessors
@@ -279,11 +313,11 @@ setAs("GenomicRanges", "GRanges",
     split0_eltNROWS <- elementNROWS(split0)
     if (S4Vectors:::anyMissingOrOutside(split0_eltNROWS, 2L, 3L))
         stop(error_msg)
-    ans_strand <- as.character(ptail(split0, n=-2L))
+    ans_strand <- as.character(tails(split0, n=-2L))
     ans_strand[is.na(ans_strand)] <- "*"
-    split1 <- phead(split0, n=2L)
-    ans_seqnames <- as.character(phead(split1, n=1L))
-    ranges <- ptail(split1, n=-1L)
+    split1 <- heads(split0, n=2L)
+    ans_seqnames <- as.character(heads(split1, n=1L))
+    ranges <- tails(split1, n=-1L)
     ranges <- setNames(as.character(ranges), names(ranges))
     ans_ranges <- try(as(ranges, "IRanges"), silent=TRUE)
     if (is(ans_ranges, "try-error"))
@@ -301,7 +335,7 @@ setAs("factor", "GRanges", .from_factor_to_GRanges)
 
 ### Does NOT propagate the ranges names and metadata columns i.e. always
 ### returns an unnamed GRanges object with no metadata columns.
-setAs("RangesList", "GRanges",
+setAs("IntegerRangesList", "GRanges",
       function(from)
       {
         if (!length(from))
@@ -358,8 +392,8 @@ setAs("RangedData", "GRanges",
 
 setAs("Seqinfo", "GRanges", .from_Seqinfo_to_GRanges)
 
-setAs("Seqinfo", "RangesList",
-    function(from) as(as(from, "GRanges"), "RangesList")
+setAs("Seqinfo", "IntegerRangesList",
+    function(from) as(as(from, "GRanges"), "IntegerRangesList")
 )
 
 setAs("ANY", "GenomicRanges", function(from) as(from, "GRanges"))
@@ -371,30 +405,10 @@ setAs("ANY", "GenomicRanges", function(from) as(from, "GRanges"))
 
 .DollarNames.GRanges <- .DollarNames.GenomicRanges
 
-setMethod("extractROWS", "GRanges",
-    function(x, i)
-    {
-        if (missing(i) || !is(i, "Ranges"))
-            i <- normalizeSingleBracketSubscript(i, x)
-        ans_seqnames <- extractROWS(seqnames(x), i)
-        ans_ranges <- extractROWS(ranges(x), i)
-        ans_strand <- extractROWS(strand(x), i)
-        ans_mcols <- extractROWS(mcols(x), i)
-        ans_ecs <- lapply(extraColumnSlots(x), extractROWS, i)
-        BiocGenerics:::replaceSlots(x, seqnames=ans_seqnames,
-                                       ranges=ans_ranges,
-                                       strand=ans_strand,
-                                       elementMetadata=ans_mcols,
-                                       .slotList=ans_ecs,
-                                       check=FALSE)
-    }
-)
-
 setMethod("replaceROWS", "GRanges",
     function(x, i, value)
     {
-        if (missing(i) || !is(i, "Ranges"))
-            i <- normalizeSingleBracketSubscript(i, x)
+        i <- normalizeSingleBracketSubscript(i, x, as.NSBS=TRUE)
         seqinfo(x) <- merge(seqinfo(x), seqinfo(value))
         ans_seqnames <- replaceROWS(seqnames(x), i, seqnames(value))
         ans_ranges <- replaceROWS(ranges(x), i, ranges(value))
@@ -406,7 +420,7 @@ setMethod("replaceROWS", "GRanges",
             ans_ecs <- NULL
         } else {
             value_ecs_names <- extraColumnSlotNames(value)
-            if (!identical(value_ecs_names[seq_len(ans_necs)],
+            if (!identical(head(value_ecs_names, n=ans_necs),
                            ans_ecs_names))
                 stop("'value' can have more extra column slots but not less")
             ans_ecs <- extraColumnSlotsAsDF(x)
@@ -421,53 +435,3 @@ setMethod("replaceROWS", "GRanges",
     }
 )
 
-### TODO: Refactor to use replaceROWS(). This will make the code much simpler
-### and avoid a lot of duplication with the above "replaceROWS" method.
-setReplaceMethod("[", "GRanges",
-    function(x, i, j, ..., value)
-    {
-        if (!is(value, "GenomicRanges"))
-            stop("replacement value must be a GenomicRanges object")
-        seqinfo(x) <- merge(seqinfo(x), seqinfo(value))
-        seqnames <- seqnames(x)
-        ranges <- ranges(x)
-        strand <- strand(x)
-        ans_mcols <- mcols(x, FALSE)
-        value_ecs <- extraColumnSlotsAsDF(value)
-        x_ecs <- extraColumnSlotsAsDF(x)
-        new_ecs <- value_ecs[!names(value_ecs) %in% names(x_ecs)]
-        ecs_to_replace <- intersect(names(value_ecs), names(x_ecs))
-        if (missing(i)) {
-            seqnames[] <- seqnames(value)
-            ranges[] <- ranges(value)
-            strand[] <- strand(value)
-            if (missing(j))
-                ans_mcols[ , ] <- mcols(value, FALSE)
-            else
-                ans_mcols[ , j] <- mcols(value, FALSE)
-            if (length(new_ecs) > 0L)
-                ans_mcols[names(new_ecs)] <- new_ecs
-            x_ecs[ecs_to_replace] <- value_ecs[ecs_to_replace]
-        } else {
-            i <- extractROWS(setNames(seq_along(x), names(x)), i)
-            seqnames[i] <- seqnames(value)
-            ranges[i] <- ranges(value)
-            strand[i] <- strand(value)
-            if (missing(j))
-                ans_mcols[i, ] <- mcols(value, FALSE)
-            else
-                ans_mcols[i, j] <- mcols(value, FALSE)
-            if (length(new_ecs) > 0L)
-                ans_mcols[i, names(new_ecs)] <- DataFrame(new_ecs)
-            if (length(ecs_to_replace) > 0L) {
-              x_ecs[i, ecs_to_replace] <- value_ecs[ecs_to_replace]
-            }
-        }
-        BiocGenerics:::replaceSlots(x, seqnames=seqnames,
-                                       ranges=ranges,
-                                       strand=strand,
-                                       elementMetadata=ans_mcols,
-                                       .slotList=as.list(x_ecs))
-    }
-)
-
