@@ -3,17 +3,26 @@
 ### -------------------------------------------------------------------------
 ###
 
+
 setClass("GRangesList",
-    contains=c("CompressedList", "GenomicRangesList"),
+    contains=c("GenomicRangesList", "CompressedRangesList"),
     representation(
-        unlistData="GRanges",
-        elementMetadata="DataFrame"
+        unlistData="GRanges"
     ),
     prototype(
         elementType="GRanges"
     )
 )
 
+### [HP] Added this on Feb 1, 2018 and made the necessary adjustments to make
+### the GRangesList() constructor return a CompressedGRangesList instance
+### instead of a GRangesList instance. The long term goal is that GRangesList
+### becomes a virtual class with CompressedGRangesList a concrete subclass
+### of it. But before we can make GRangesList virtual, we need to make sure
+### that all the existing serialized GRangesList instances are replaced with
+### CompressedGRangesList instances. This might take a while!
+setClass("CompressedGRangesList", contains="GRangesList")
+
 ### Note that rtracklayer also defines GenomicRanges_OR_GenomicRangesList.
 ### Do we need the 2 union classes?
 setClassUnion("GenomicRanges_OR_GRangesList", c("GenomicRanges", "GRangesList"))
@@ -49,6 +58,55 @@ setClassUnion("GenomicRanges_OR_GRangesList", c("GenomicRanges", "GRangesList"))
 setValidity2("GRangesList", .valid.GRangesList)
 
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### updateObject()
+###
+### Starting with GenomicRanges 1.31.13, all GRangesList instances need to be
+### replaced with CompressedGRangesList instances. Note that this NOT a change
+### of the internals (GRangesList instances have been using the CompressedList
+### representation since the begining), only a change of the class attribute.
+###
+
+setMethod("updateObject", "GRangesList",
+    function(object, ..., verbose=FALSE)
+    {
+        ## class attribute.
+        if (class(object) != "GRangesList") {
+            if (verbose)
+                message("[updateObject] ", class(object), " object ",
+                        "is current.\n",
+                        "[updateObject] Nothing to update.")
+        } else {
+            if (verbose)
+                message("[updateObject] class attribute of ", class(object),
+                        " object needs to be set to ",
+                        "\"CompressedGRangesList\"\n",
+                        "[updateObject] Updating it ...")
+            class(object) <- class(new("CompressedGRangesList"))
+        }
+        ## unlistData slot.
+        object@unlistData <- updateObject(object@unlistData,
+                                          ..., verbose=verbose)
+        ## Call method for CompressedList to update partitioning slot.
+        callNextMethod()
+    }
+)
+
+### Overwrite method for CompressedList objects just so we can fix on-the-fly
+### any old GRanges instance stuck in the 'unlistData' slot.
+setMethod("unlist", "GRangesList",
+    function(x, recursive=TRUE, use.names=TRUE)
+    {
+        if (!isTRUEorFALSE(use.names))
+            stop("'use.names' must be TRUE or FALSE")
+        unlisted_x <- updateObject(x@unlistData)
+        if (use.names)
+            unlisted_x <- S4Vectors:::set_unlisted_names(unlisted_x, x)
+        unlisted_x
+    }
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Constructors.
 ###
@@ -57,7 +115,7 @@ GRangesList <- function(...)
 {
     listData <- list(...)
     if (length(listData) == 1L && !is(listData[[1L]], "GRanges"))
-        return(as(listData[[1L]], "GRangesList"))
+        return(as(listData[[1L]], "CompressedGRangesList", strict=FALSE))
     if (length(listData) == 0L) {
         unlistData <- GRanges()
     } else {
@@ -132,20 +190,6 @@ makeGRangesListFromFeatureFragments <- function(seqnames=Rle(factor()),
     relist(unlistData, partitioning)
 }
 
-setMethod("updateObject", "GRangesList",
-    function(object, ..., verbose=FALSE)
-    {
-        if (verbose)
-            message("updateObject(object = 'GRangesList')")
-        if (is(try(validObject(object@unlistData, complete=TRUE), silent=TRUE),
-               "try-error")) {
-            object@unlistData <- updateObject(object@unlistData)
-            return(object)
-        }
-        object
-    }
-)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Accessors.
@@ -199,24 +243,15 @@ setMethod("ranges", "GRangesList",
 setReplaceMethod("ranges", "GRangesList",
     function(x, value)
     {
-        if (!is(value, "RangesList") ||
+        if (!is(value, "IntegerRangesList") ||
             !identical(elementNROWS(x), elementNROWS(value)))
-            stop("replacement 'value' is not a RangesList with the same ",
-                 "elementNROWS as 'x'")
+            stop("replacement 'value' is not an IntegerRangesList with the ",
+                 "same elementNROWS as 'x'")
         ranges(x@unlistData) <- as(unlist(value, use.names = FALSE), "IRanges")
         x
     }
 )
 
-### Same as for CompressedIRangesList.
-setMethod("start", "GRangesList",
-    function(x, ...)
-    {
-        unlisted_x <- unlist(x, use.names=FALSE)
-        relist(start(unlisted_x), x)
-    }
-)
-
 setReplaceMethod("start", "GRangesList",
     function(x, ..., value)
     {
@@ -230,15 +265,6 @@ setReplaceMethod("start", "GRangesList",
     }
 )
 
-### Same as for CompressedIRangesList.
-setMethod("end", "GRangesList",
-    function(x, ...)
-    {
-        unlisted_x <- unlist(x, use.names=FALSE)
-        relist(end(unlisted_x), x)
-    }
-)
-
 setReplaceMethod("end", "GRangesList",
     function(x, ..., value)
     {
@@ -252,15 +278,6 @@ setReplaceMethod("end", "GRangesList",
     }
 )
 
-### Same as for CompressedIRangesList.
-setMethod("width", "GRangesList",
-    function(x)
-    {
-        unlisted_x <- unlist(x, use.names=FALSE)
-        relist(width(unlisted_x), x)
-    }
-)
-
 setReplaceMethod("width", "GRangesList",
     function(x, ..., value)
     {
@@ -422,14 +439,34 @@ setAs("GRangesList", "CompressedIRangesList",
 setAs("GRangesList", "IRangesList",
     function(from) ranges(from, use.mcols=TRUE)
 )
-setAs("GRangesList", "RangesList",
+setAs("GRangesList", "IntegerRangesList",
     function(from) ranges(from, use.mcols=TRUE)
 )
 
-setAs("GRanges", "GRangesList", function(from) as(from, "List"))
+.from_GenomicRanges_to_CompressedGRangesList <- function(from)
+{
+    if (!is(from, "GRanges"))
+        from <- as(from, "GRanges", strict = FALSE)
+    ans_partitioning <- PartitioningByEnd(seq_along(from), names=names(from))
+    names(from) <- NULL
+    ans_mcols <- mcols(from)
+    mcols(from) <- NULL
+    ans <- relist(from, ans_partitioning)
+    mcols(ans) <- ans_mcols
+    ans
+}
+setAs("GenomicRanges", "CompressedGRangesList",
+    .from_GenomicRanges_to_CompressedGRangesList
+)
+setAs("GenomicRanges", "GRangesList",
+    .from_GenomicRanges_to_CompressedGRangesList
+)
 
 setAs("list", "GRangesList",
       function(from) do.call(GRangesList, from))
+setAs("list", "CompressedGRangesList",
+      function(from) do.call(GRangesList, from))
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting.
@@ -489,31 +526,12 @@ setMethod("[", "GRangesList", .sBracketSubsetGRList)
 }
 setReplaceMethod("[", "GRangesList", .sBracketReplaceGRList)
 
-.dBracketReplaceGRList <- function(x, i, j, ..., value)
-{
-    nameValue <- if (is.character(i)) i else ""
-    i <- S4Vectors:::normargSubset2_iOnly(x, i, j, ...,
-             .conditionPrefix=paste0("[[<-,", class(x)[1], "-method: "))
-    len <- length(x)
-    if (i > len) {
-        value <- list(value)
-        if (nzchar(nameValue))
-            names(value) <- nameValue
-        x <- c(x, do.call(getFunction(class(x)), value))
-    } else {
-        x <- callNextMethod(x, i, ..., value=value)
-    }
-    validObject(x)
-    x
-}
-setReplaceMethod("[[", "GRangesList", .dBracketReplaceGRList)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Going from GRanges to GRangesList with extractList() and family.
 ###
 
-setMethod("relistToClass", "GRanges", function(x) "GRangesList")
+setMethod("relistToClass", "GRanges", function(x) "CompressedGRangesList")
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -521,13 +539,21 @@ setMethod("relistToClass", "GRanges", function(x) "GRangesList")
 ###
 
 ### NOT exported but used in GenomicAlignments package.
-### FIXME: This seems to repeat most of the code in IRanges:::showRangesList!
+### FIXME: This seems to repeat most of the code in IRanges:::show_IRangesList!
 showList <- function(object, showFunction, print.classinfo)
 {
+    if (class(object) == "GRangesList") {
+        warning(wmsg("Note that starting with BioC 3.7, the class attribute ",
+                     "of all GRangesList **instances** needs to be set to ",
+                     "\"CompressedGRangesList\". Please update this object ",
+                     "with 'updateObject(object, verbose=TRUE)' and ",
+                     "re-serialize it."))
+        object <- updateObject(object)
+    }
     k <- length(object)
     cumsumN <- cumsum(elementNROWS(object))
     N <- tail(cumsumN, 1)
-    cat(class(object), " object of length ", k, ":\n", sep = "")
+    cat(classNameForDisplay(object), " object of length ", k, ":\n", sep = "")
     if (k == 0L) {
         cat("<0 elements>\n\n")
     } else if ((k == 1L) || ((k <= 3L) && (N <= 20L))) {
