@@ -6,11 +6,12 @@
 ### TODO: The 'constraint' slot could be moved to the Vector class (or to the
 ### Annotated class) so any Vector object could be constrained.
 setClass("GenomicRanges",
-    contains="Vector",
+    contains="Ranges",
     representation(
-        "VIRTUAL"#,
+        "VIRTUAL",
         #No more constraint slot for now...
-        #constraint="Constraint_OR_NULL"
+        #constraint="Constraint_OR_NULL",
+        elementMetadata="DataFrame"
     )
 )
 
@@ -20,12 +21,19 @@ setClassUnion("GenomicRanges_OR_missing", c("GenomicRanges", "missing"))
 ### seqnames(x), ranges(x), strand(x), seqlengths(x), seqinfo(),
 ### and update(x) are defined.
 
+setClass("GenomicPos",
+    contains=c("GenomicRanges", "Pos"),
+    representation("VIRTUAL")
+)
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Getters.
 ###
 
-setMethod("length", "GenomicRanges", function(x) length(seqnames(x)))
+### The method for Ranges does 'length(start(x))' which is very inefficient
+### on GPos objects so we overwrite it with a faster method.
+setMethod("length", "GenomicRanges", function(x) length(ranges(x)))
 
 setMethod("names", "GenomicRanges", function(x) names(ranges(x)))
 
@@ -33,23 +41,41 @@ setMethod("names", "GenomicRanges", function(x) names(ranges(x)))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Extra column slots (implemented by subclasses)
+### Extra column slots (added by GRanges subclasses)
+###
+### The "extra column slots" are parallel slots added by GRanges subclasses
+### in addition to GRanges parallel slots "seqnames", "ranges", "strand",
+### and "elementMetadata".
+###
+### TODO: Packages defining "extraColumnSlotNames" methods (e.g.
+### VariantAnnotation, GenomicTuples, InteractionSet, SGSeq) should define
+### "parallelSlotNames" methods instead. They'll get more things working
+### out-of-the-box by doing so (e.g. c()). Then the extraColumnSlotNames()
+### generic will become a GenomicRanges private business and could be made
+### a regular function.
 ###
 
-extraColumnSlots <- function(x) {
-  sapply(extraColumnSlotNames(x), slot, object = x, simplify = FALSE)
-}
-
-extraColumnSlotsAsDF <- function(x) {
-  ## low-level fast path; otherwise, would need to wrap some things with I()
-  new("DataFrame", listData = extraColumnSlots(x), nrows = length(x))
-}
-
 setGeneric("extraColumnSlotNames",
-           function(x) standardGeneric("extraColumnSlotNames"))
+    function(x) standardGeneric("extraColumnSlotNames")
+)
 
 setMethod("extraColumnSlotNames", "ANY", function(x) character())
 
+#setMethod("extraColumnSlotNames", "GRanges",
+#    function(x)
+#    {
+#        GRanges_pslotnames <- parallelSlotNames(new("GRanges"))
+#        setdiff(parallelSlotNames(x), GRanges_pslotnames)
+#    }
+#)
+
+extraColumnSlots <- function(x)
+    sapply(extraColumnSlotNames(x), slot, object=x, simplify=FALSE)
+
+extraColumnSlotsAsDF <- function(x)
+    S4Vectors:::new_DataFrame(extraColumnSlots(x), nrows=length(x))
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### 2 low-level helper functions to deal with out-of-bound ranges.
 ###
@@ -86,11 +112,10 @@ make_out_of_bound_warning_msg <- function(x, idx, suggest.trim)
     msg <- c(class(x), " object contains ", length(idx), " out-of-bound ",
              "range", if (length(idx) >= 2L) "s" else "", " located on ",
              on_what, ". ",
-             "Note that only ranges located on a non-circular ",
-             "sequence whose length is not NA can be considered ",
-             "out-of-bound (use seqlengths() and isCircular() to ",
-             "get the lengths and circularity flags of the underlying ",
-             "sequences).")
+             "Note that ranges located on a sequence whose length is unknown ",
+             "(NA) or on a circular sequence are not considered out-of-bound ",
+             "(use seqlengths() and isCircular() to get the lengths and ",
+             "circularity flags of the underlying sequences).")
     if (suggest.trim)
         msg <- c(msg, " You can use trim() to trim these ranges. ",
                  "See ?`trim,GenomicRanges-method` for more information.")
@@ -196,14 +221,6 @@ valid.GenomicRanges.seqinfo <- function(x, suggest.trim=FALSE)
     NULL
 }
 
-## For convenience, validate the extra column slots that are virtual
-## classes. Since they are not directly constructed, any validity
-## checks specific to the virtual class have probably not been called.
-.valid.GenomicRanges.ecs <- function(x) {
-  virtuals <- Filter(isVirtualClass, getSlots(class(x))[extraColumnSlotNames(x)])
-  unlist(lapply(names(virtuals), function(nm) validObject(slot(x, nm))))
-}
-
 .valid.GenomicRanges <- function(x)
 {
     c(.valid.GenomicRanges.length(x),
@@ -211,8 +228,7 @@ valid.GenomicRanges.seqinfo <- function(x, suggest.trim=FALSE)
       .valid.GenomicRanges.ranges(x),
       .valid.GenomicRanges.strand(x),
       .valid.GenomicRanges.mcols(x),
-      valid.GenomicRanges.seqinfo(x, suggest.trim=TRUE),
-      .valid.GenomicRanges.ecs(x))
+      valid.GenomicRanges.seqinfo(x, suggest.trim=TRUE))
       #checkConstraint(x, constraint(x)))
 }
 
@@ -279,23 +295,31 @@ setMethod("as.data.frame", "GenomicRanges",
     }
 )
 
-setAs("GenomicRanges", "RangesList",
-    function(from)
-    {
-        strand_mcols <- DataFrame(strand=strand(from), mcols(from))
-        ecs <- extraColumnSlotsAsDF(from)
-        if (length(ecs))
-          strand_mcols <- cbind(strand_mcols, ecs)
-        rngs <- ranges(from)
-        mcols(rngs) <- strand_mcols
-        rl <- split(rngs, seqnames(from))
-        mcols(rl) <- DataFrame(seqlengths=seqlengths(from),
-                               isCircular=isCircular(from),
-                               genome=genome(from))
-        metadata(rl) <- metadata(from)
-        metadata(rl)$seqinfo <- seqinfo(from)
-        rl
-    }
+.from_GenomicRanges_to_CompressedIRangesList <- function(from)
+{
+    strand_mcols <- DataFrame(strand=strand(from), mcols(from))
+    ecs <- extraColumnSlotsAsDF(from)
+    if (length(ecs))
+      strand_mcols <- cbind(strand_mcols, ecs)
+    rngs <- ranges(from)
+    mcols(rngs) <- strand_mcols
+    ans <- split(rngs, seqnames(from))
+    mcols(ans) <- DataFrame(seqlengths=seqlengths(from),
+                           isCircular=isCircular(from),
+                           genome=genome(from))
+    metadata(ans) <- metadata(from)
+    metadata(ans)$seqinfo <- seqinfo(from)
+    ans
+}
+
+setAs("GenomicRanges", "CompressedIRangesList",
+    .from_GenomicRanges_to_CompressedIRangesList
+)
+setAs("GenomicRanges", "IRangesList",
+    .from_GenomicRanges_to_CompressedIRangesList
+)
+setAs("GenomicRanges", "IntegerRangesList",
+    .from_GenomicRanges_to_CompressedIRangesList
 )
 
 setAs("GenomicRanges", "RangedData",
@@ -324,8 +348,8 @@ setAs("GenomicRanges", "RangedData",
 setReplaceMethod("names", "GenomicRanges",
     function(x, value)
     {
-        names(ranges(x)) <- value
-        x
+        x_ranges <- setNames(ranges(x), value)
+        update(x, ranges=x_ranges, check=FALSE)
     }
 )
 
@@ -413,6 +437,8 @@ setReplaceMethod("seqinfo", "GenomicRanges", set_GenomicRanges_seqinfo)
 
 setMethod("score", "GenomicRanges", function(x) mcols(x)$score)
 setReplaceMethod("score", "GenomicRanges", function(x, value) {
+  ## Fix old GRanges instances on-the-fly.
+  x <- updateObject(x)
   mcols(x)$score <- value
   x
 })
@@ -456,11 +482,10 @@ setMethod("clone", "ANY",  # not exported
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Ranges methods.
+### Range accessors
 ###
 
 setMethod("start", "GenomicRanges", function(x, ...) start(ranges(x)))
-setMethod("end", "GenomicRanges", function(x, ...) end(ranges(x)))
 setMethod("width", "GenomicRanges", function(x) width(ranges(x)))
 
 setReplaceMethod("start", "GenomicRanges",
@@ -492,21 +517,6 @@ setReplaceMethod("width", "GenomicRanges",
 ### Subsetting.
 ###
 
-### Needed only because we want to support x[i, j] subsetting.
-setMethod("[", "GenomicRanges",
-    function(x, i, j, ..., drop)
-    {
-        if (length(list(...)) > 0L)
-            stop("invalid subsetting")
-        if (!missing(i))
-            x <- extractROWS(x, i)
-        if (missing(j))
-            return(x)
-        new_mcols <- mcols(x)[ , j, drop=FALSE]
-        clone(x, elementMetadata=new_mcols, check=FALSE)
-    }
-)
-
 ### Subset a named list-like object *by* a GenomicRanges subscript.
 ### The returned object 'ans' is as follow:
 ###   (a) 'ans' is parallel to 'gr'.
@@ -608,50 +618,12 @@ setMethod("$", "GenomicRanges",
 )
 
 setReplaceMethod("$", "GenomicRanges",
-    function(x, name, value) {mcols(x)[[name]] <- value; x}
-)
-
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Combining.
-###
-
-### Not exported. 'x' *must* be an unnamed list of length >= 1 (not checked).
-.unlist_list_of_GenomicRanges <- function(x, ignore.mcols=FALSE)
-{
-    if (!isTRUEorFALSE(ignore.mcols))
-        stop("'ignore.mcols' must be TRUE or FALSE")
-    ans_class <- class(x[[1L]])
-    ans_seqinfo <- do.call(merge, lapply(x, seqinfo))
-    ans_seqnames <- do.call(c, lapply(x, seqnames))
-    ans_ranges <- do.call(c, lapply(x, ranges))
-    ans_strand <- do.call(c, lapply(x, strand))
-    if (ignore.mcols) {
-        ans_mcols <- S4Vectors:::make_zero_col_DataFrame(length(ans_ranges))
-    } else {
-        ans_mcols <- do.call(rbind, lapply(x, mcols, FALSE))
-    }
-    if (length(extraColumnSlotNames(x[[1L]])) > 0L) {
-        ans_ecs <- do.call(rbind, lapply(x, extraColumnSlotsAsDF))
-        do.call(new, c(list(ans_class, seqnames=ans_seqnames, ranges=ans_ranges,
-                            strand=ans_strand, elementMetadata=ans_mcols,
-                            seqinfo=ans_seqinfo), as.list(ans_ecs)))
-    } else {
-        new(ans_class, seqnames=ans_seqnames, ranges=ans_ranges,
-            strand=ans_strand, elementMetadata=ans_mcols, seqinfo=ans_seqinfo)
-    }
-}
-
-setMethod("c", "GenomicRanges",
-    function(x, ..., ignore.mcols=FALSE, recursive=FALSE)
+    function(x, name, value)
     {
-        if (!identical(recursive, FALSE))
-            stop("'recursive' argument not supported")
-        if (missing(x))
-            args <- unname(list(...))
-        else
-            args <- unname(list(x, ...))
-        .unlist_list_of_GenomicRanges(args, ignore.mcols=ignore.mcols)
+        ## Fix old GRanges instances on-the-fly.
+        x <- updateObject(x)
+        mcols(x)[[name]] <- value
+        x
     }
 )
 
@@ -750,5 +722,29 @@ setMethod("show", "GenomicRanges",
                            print.classinfo=TRUE, print.seqinfo=TRUE)
 )
 
-setMethod("showAsCell", "GenomicRanges", function(object) as.character(object))
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Concatenation
+###
+
+### NOT exported but used in the GenomicAlignments package.
+concatenate_GenomicRanges_objects <-
+    function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
+{
+    ## Fix old GRanges and GAlignments instance on-the-fly.
+    x <- updateObject(x, check=FALSE)
+
+    objects <- S4Vectors:::prepare_objects_to_concatenate(x, objects)
+    all_objects <- c(list(x), objects)
+
+    ## Combine seqinfo.
+    seqinfo(x) <- do.call(merge, lapply(all_objects, seqinfo))
+
+    ## Call method for Vector objects to concatenate all the parallel slots.
+    callNextMethod()
+}
+
+setMethod("concatenateObjects", "GenomicRanges",
+    concatenate_GenomicRanges_objects
+)
 
